#include <avr/io.h>
#include <avr/interrupt.h>
#include "hal_mlx90609.h"

#define HAL_TRX_CMD_RW         (0x00) /**<  Register Write (short mode). */
#define HAL_TRX_CMD_RR         (0x80) /**<  Register Read (short mode). */
#define HAL_TRX_CMD_RADDRM     (0x7F) /**<  Register Address Mask. */


/*----------------------------------------------------------------------------*/
/** \brief  This function starts an ADC conversion of the angular sensor.
 *			See datasheet.
 * 
 *  \param  chipSelect Chip Select-Address of the mlx90609.
 *
 *  \returns 	true (1) if the instruction was accepted.
 *				false (0) if the function fails.
 */
int8_t hal_mlx90609_start_angular_conversion( uint8_t chipSelect){

    uint8_t register_value = 0;
	uint8_t dummy_read;
	uint8_t dummy_write = 0xFF;

    AVR_ENTER_CRITICAL_REGION();

	if( CS1 == chipSelect)
		HAL_SET_CS1_LOW();
	if( CS2 == chipSelect)
		HAL_SET_CS2_LOW();
	if( CS3 == chipSelect)
		HAL_SET_CS3_LOW();

	/*Send conversion start instruction.*/
    SPDR = 0x94;
    while ((SPSR & (1 << SPIF)) == 0) {;}
    dummy_read = SPDR;

	/*Read the answer.*/
    SPDR = dummy_write;
    while ((SPSR & (1 << SPIF)) == 0) {;}
    register_value = SPDR;

	HAL_SET_CS1_HIGH();
	HAL_SET_CS2_HIGH();
	HAL_SET_CS3_HIGH();

	AVR_LEAVE_CRITICAL_REGION();

	/*Return true when the instruction was accepted.*/
	if( !(register_value & 0x80)){
		return 1;
	}

	return 0;
}


/*----------------------------------------------------------------------------*/
/** \brief  This function reads the ADC result of the mlx90609.
 *			See datasheet.
 * 
 *  \param  chipSelect Chip Select-Address of the mlx90609.
 *	\param	*ptr_value Adress where the value should be stored.
 *
 *  \returns 	true (1) if the ADC result is valid.
 *				false (0) if the ADC result is not valid.
 */
int8_t hal_mlx90609_read_conversion_result( uint8_t chipSelect, uint16_t *ptr_value){

   	uint16_t low_value=0;
	uint16_t high_value=0;
	uint16_t value=0;

	uint8_t dummy_read;
	uint8_t dummy_write = 0xFF;

	AVR_ENTER_CRITICAL_REGION();

	if( CS1 == chipSelect)
		HAL_SET_CS1_LOW();
	if( CS2 == chipSelect)
		HAL_SET_CS2_LOW();
	if( CS3 == chipSelect)
		HAL_SET_CS3_LOW();

	/*Send ADC reading instruction.*/
    SPDR = 0x80;
    while ((SPSR & (1 << SPIF)) == 0) {;}
    dummy_read = SPDR;

	/*Read the high byte of the answer.*/
    SPDR = dummy_write;
    while ((SPSR & (1 << SPIF)) == 0) {;}
    high_value = SPDR;

	/*Read the low byte of the answer.*/
    SPDR = dummy_write;
    while ((SPSR & (1 << SPIF)) == 0) {;}
    low_value = SPDR;

	HAL_SET_CS1_HIGH();
	HAL_SET_CS2_HIGH();
	HAL_SET_CS3_HIGH();

	AVR_LEAVE_CRITICAL_REGION();

	/*Return true when the instruction was accepted.*/
	if( high_value & 0x0020){
	
		high_value = high_value << 8;
	
		value = high_value | low_value;

		value = value >> 1;

		value = value & 0x07FF;

		*ptr_value = value;
	
		return 1;
	}
	
	return 0;
}